"""Models for putting the input of the desired content"""

from typing import Optional

from pydantic import BaseModel, field_validator

from scaffoldry.models_pkg.enum_values import (
    FrameworkUsed,
    ProgrammingLanguage,
    RelationshipValue,
)


class Author(BaseModel):
    first_name: Optional[str] = ""
    middle_name: Optional[str] = ""
    last_name: Optional[str] = ""
    email: Optional[str] = ""


class Version(BaseModel):
    major: int
    minor: int
    patch: int


class VersionLanguage(Version):
    relationship: RelationshipValue = RelationshipValue.EQUAL


class VersionRelease(Version):
    finalacronym: Optional[str] = None


class LanguageEnvironment(BaseModel):
    programming_language: ProgrammingLanguage
    framework_language: Optional[FrameworkUsed] = None
    version_language: Optional[VersionLanguage] = None


class BaseTemplate(BaseModel):
    language_environment: LanguageEnvironment
    description: str
    package_name: str
    repository_name: str
    author_repository: str
    author: Author
    version: Version

    @field_validator("package_name")
    def check_spaces(cls, in_str: str) -> str:
        if " " in in_str:
            raise ValueError("there are spaces into package name")
        return in_str



import json
from pathlib import Path

import typer

from scaffoldry.generators.core import prepare_project
from scaffoldry.init_template_objs.python_pkg import create_model
from scaffoldry.main_arguments import CommandAppGenerate, CommandAppCreate
from scaffoldry.models_pkg.models import BaseTemplate

app = typer.Typer()


@app.command()
def generate(
    path_definition: str = CommandAppGenerate.path_definition,
    path_location: str = CommandAppGenerate.path_location,
) -> None:
    """Generate all the content related with the input file provided"""
    with open(path_definition, "r") as f_r:
        model_obj = json.load(f_r)
    prepare_project(Path(path_location), BaseTemplate.model_validate(model_obj))


@app.command()
def create_template(
    path_loc: str = CommandAppCreate.path_loc,
) -> None:
    """Create a template in json in order to modify it."""
    var_model = create_model()
    Path(path_loc).parent.mkdir(parents=True, exist_ok=True)
    Path(path_loc).write_text(var_model.model_dump_json(indent=2))


def main() -> None:
    app()


if __name__ == "__main__":
    main()



from enum import Enum


class RelationshipValue(str, Enum):
    EQUAL = "="
    NOT_EQUAL = "!="
    MORE_THAN = ">"
    LESS_THAN = "<"
    MORE_OR_EQUAL_THAN = ">="
    LESS_OR_EQUAL_THAN = "<="


class FrameworkUsed(str, Enum):
    DJANGO = "django"
    FASTAPI = "fastapi"


class ProgrammingLanguage(str, Enum):
    """Programming language in which we want to generate the template."""

    PYTHON = "python"



from scaffoldry.generators.pygen import prepare_project_python
from scaffoldry.models_pkg.enum_values import ProgrammingLanguage
from scaffoldry.models_pkg.models import BaseTemplate
from pathlib import Path


def prepare_project(root_location: Path, in_project: BaseTemplate) -> None:
    if (
        in_project.language_environment.programming_language
        is ProgrammingLanguage.PYTHON
    ):
        prepare_project_python(root_location, in_project)



from pathlib import Path

from scaffoldry.templates import core


current_folder = Path(__file__).parent.relative_to(core._path)
pyproject_toml_j2_path = current_folder / "pyproject.toml.j2"
gitignore_j2_path = current_folder / ".gitignore.j2"
checks_sh_j2_path = current_folder / "checks.sh.j2"
ci_yml_j2 = current_folder / "ci.yml.j2"
release_yml_j2 = current_folder / "release.yml.j2"
flake8_j2 = current_folder / ".flake8.j2"
package_base_name = current_folder / "main.py.j2"
pre_commit_config_yaml_j2 = current_folder / ".pre-commit-config.yaml.j2"



from scaffoldry.models_pkg.enum_values import ProgrammingLanguage
from scaffoldry.models_pkg.models import (
    BaseTemplate,
    LanguageEnvironment,
    VersionLanguage,
    Author,
)


def create_model() -> BaseTemplate:
    return BaseTemplate(
        repository_name="packagerepository",
        author_repository="author",
        language_environment=LanguageEnvironment(
            programming_language=ProgrammingLanguage.PYTHON,
            version_language=VersionLanguage(major=3, minor=10, patch=1),
        ),
        description="Put here your definition of package",
        package_name="packagename",
        author=Author(
            first_name="firstname",
            last_name="lastname",
            middle_name="middlename",
            email="yourmail@domain.com",
        ),
        version=VersionLanguage(major=0, minor=0, patch=0),
    )



from pathlib import Path
from typing import Optional, List

from jinja2 import Environment, FileSystemLoader, Template
from pydantic import BaseModel

from scaffoldry.models_pkg.models import BaseTemplate


def get_template(in_path_template: Path) -> Template:
    return _env.get_template(in_path_template.as_posix())


def render_template(
    in_base_obj: BaseTemplate, in_path_template: Path, out_path: Path
) -> None:
    out_path.parent.mkdir(parents=True, exist_ok=True)
    out_path.open("w").write(
        get_template(in_path_template).render(in_base_obj.model_dump())
    )


def final_basename(template_base: Path) -> str:
    return template_base.name.replace(".j2", "")


def generate_final_file(
    in_base_obj: BaseTemplate,
    in_path_template: Path,
    out_path_folder: Path,
    out_base_name: Optional[str] = None,
) -> None:
    if out_base_name is None:
        out_base_name = final_basename(in_path_template)

    render_template(
        in_base_obj,
        in_path_template,
        out_path_folder / out_base_name,
    )


class TemplateConfigElement(BaseModel):
    in_path_template: Path
    out_relative_folder: Path = Path("")
    out_base_name: Optional[str] = None


def generate_final_files(
    in_base_obj: BaseTemplate,
    common_root_location: Path,
    list_templates_config_elements: List[TemplateConfigElement],
) -> None:
    for in_template in list_templates_config_elements:
        if in_template.out_base_name is None:
            in_template.out_base_name = final_basename(
                in_template.in_path_template
            )

        render_template(
            in_base_obj,
            in_template.in_path_template,
            common_root_location
            / in_template.out_relative_folder
            / in_template.out_base_name,
        )


_path = Path(__file__).parent
_env = Environment(loader=FileSystemLoader(_path.as_posix()))



from pathlib import Path

from scaffoldry.models_pkg.models import BaseTemplate
from scaffoldry.templates.pypackage import paths
from scaffoldry.templates.core import (
    TemplateConfigElement,
    generate_final_files,
)


def prepare_project_python(
    root_location: Path, in_project: BaseTemplate
) -> None:
    """Prepare all the files from a project in python."""
    root_location.mkdir(parents=True, exist_ok=True)

    generate_final_files(
        in_project,
        root_location,
        [
            TemplateConfigElement(
                in_path_template=paths.pyproject_toml_j2_path
            ),
            TemplateConfigElement(in_path_template=paths.gitignore_j2_path),
            TemplateConfigElement(
                in_path_template=paths.pre_commit_config_yaml_j2
            ),
            TemplateConfigElement(
                in_path_template=paths.checks_sh_j2_path,
                out_relative_folder=Path("scripts"),
            ),
            TemplateConfigElement(
                in_path_template=paths.ci_yml_j2,
                out_relative_folder=Path(".github", "workflows"),
            ),
            TemplateConfigElement(
                in_path_template=paths.release_yml_j2,
                out_relative_folder=Path(root_location, ".github", "workflows"),
            ),
            TemplateConfigElement(
                in_path_template=paths.flake8_j2,
            ),
            TemplateConfigElement(
                in_path_template=paths.package_base_name,
                out_relative_folder=Path(
                    root_location, "src", in_project.package_name
                ),
                out_base_name="main.py",
            ),
        ],
    )
    Path(root_location, "src", in_project.package_name, "__init__.py").touch()



import typer


class CommandAppGenerate:
    path_definition = typer.Argument(..., help="Path with the definition file")
    path_location = typer.Argument(
        ..., help="Output root path in which the project is generated"
    )


class CommandAppCreate:
    path_loc = typer.Argument(help="Path to store the template")
